/*
    Copyright (c) 1997 Netscape Communications Corporation, Inc. All Rights Reserved.

    This software is the confidential and proprietary information of Netscape
    Communications Corporation.  ("Confidential Information").  You shall not
    disclose such Confidential Information and shall use it only in
    accordance with the terms of the agreement you entered into with Netscape.

    Netscape makes no representations or warranties about the suitability of
    this software.  Netscape will not be liable for any damages related to using,
    modifying or distributing this software or derivatives thereof.

    This software is provided for example purposes only.

    created: 2/97 cls
 */

package netscape.peas;




/**
 * The RowChangeSupport is a helper class to which classes which implement
 * fire the RowChangeEvent may delegate their implementations of:
 *    addRowChangeListenter
 *    removeRowChangeListener
 *    fireRowChange
 * It also implements propegateRowChange
 */
public class RowChangeSupport  {

    /**
     * Construct a RowChangeSupport class
     */
    public RowChangeSupport( Object oDelegator ) {
        moDelegator = oDelegator;
    }

    /**
     * Add an object which is a RowChangeListener to set of registered listeners.
     */
    public synchronized void addRowChangeListener( RowChangeListener oListener ) {

        if ( null == mvRowChangeListeners ) {
            mvRowChangeListeners = new java.util.Vector();
        }
        mvRowChangeListeners.addElement( oListener );

    } // addRowChangeListener


    /**
     * Remove oListener from set of registered listeners.
     */
    public synchronized void removeRowChangeListener( RowChangeListener oListener ) {
	    if ( null != mvRowChangeListeners ) {
	        mvRowChangeListeners.removeElement( oListener );
	    }

    } // removePropertyChangeListener


    /**
     * Given an event send it to all our listeners who have not heard of it yet.
     */
    public void propegateRowChange( RowChangeEvent oEvent ) {
        try {
         	java.util.Vector vTargets;
        	synchronized (this) {
        	    if ( mvRowChangeListeners == null) {
        	    	return;
        	    }
        	    vTargets = (java.util.Vector) mvRowChangeListeners.clone();
        	}

        	for ( int i = 0; i < vTargets.size(); i++ ) {
        	    RowChangeListener oTarget = (RowChangeListener)vTargets.elementAt( i );
        	    if ( ! oEvent.isOnVisitedList( oTarget ) ) {
        	        oEvent.addToVisitedList( oTarget );
            	    oTarget.rowChange( oEvent );
            	}
            }
        } catch ( Exception ex ) {
            System.out.println( "Caught exception in RowChangeSupport.propegateRowChange: " + ex );
        }
    }


    /**
     * Fire a RowChageEvent out to all our registered listeners.
     */
    public void fireRowChange( int iChangeType, String sColumnAffected, Object oOldValue, Object oNewValue, RowProvider oRowProvider, NotifyList oNotify ) {

        RowChangeEvent oEvent = new RowChangeEvent( iChangeType, sColumnAffected, oOldValue, oNewValue, oRowProvider );
        oEvent.setNotifyList( oNotify );
        oEvent.addToVisitedList( moDelegator );
        propegateRowChange( oEvent );

    }


    //-------------------------------------------------------------------------
    // Member variables

    private java.util.Vector mvRowChangeListeners;
    private Object moDelegator;

} // RowChangeSupport
